; THE "Z80 CHECK" PROGRAM LISTING
;
; Z80 CHECK
; 
; Intended for MCM Design's BalcheckHR board
; A MCM Design program
; Nov 2018
; 
; This program was added to MCM Design's BalcheckHR package in the form of a
; multi-carted 32KB EEPROM. The program was loaded into the EEPROM's 8KB
; Bank 2.
; 
; This program was hand written and "texted in" using Windows Notepad.
;

FIRSTC  EQU     $2000             ; FIRST address in Cartridge
LDSPLY  EQU     $EF               ; Left Display
RDSPLY  EQU     $FF               ; Right Display

        ORG     0
        
        NOP
        DI
        JP      PGM
; 
; ++++++++++++++++++++++++++++++++++++++++++
; 
; COLOR TABLE
;
CTBL:
        DB      $AC               ; green   pixel 11, left colors
        DB      $86               ; yellow        10
        DB      $07               ; white         01
        DB      $00               ; black         00
        DB      $CD               ; cyan    pixel 11, right colors
        DB      $79               ; brown         10
        DB      $2B               ; magenta       01
        DB      $F9               ; blue          00
; 
; +++++++++++++++++++++++++++++++++++++++++
; 
; 7-SEGMENT LED DISPLAY DIGIT DRIVERS (0-F)
; 
; Bits    7 6 5 4 3 2 1 0 
; Segment   g f e d c b a
; 
DTBL:                             ; displays
        DB      $3F               ; 0
        DB      $06               ; 1
        DB      $5B               ; 2
        DB      $4F               ; 3
        DB      $66               ; 4
        DB      $6D               ; 5
        DB      $7D               ; 6
        DB      $07               ; 7
        DB      $7F               ; 8
        DB      $67               ; 9
        DB      $77               ; A
        DB      $7C               ; b
        DB      $39               ; C
        DB      $5E               ; d
        DB      $79               ; E
        DB      $71               ; F
; 
; Notes:Hex B=b, lower case b looks similar to 6
;       Hex D=d
; 
; ++++++++++++++++++++++++++++++++++++++++++
; 
; DISPLAY DIGITS
; 
; Enter with:  A = double digit counter to display
;             IX = program "continue address" to jump to
; 
; Note registers B, C and D are clobbered
; 
DSPLY:
        LD      BC,DTBL           ; point BC at digit drivers table
        LD      D,A               ; save "counter to display"
        AND     11110000B         ; A = most significant digit
        RRCA                      ; rotate left nibble right to bits 3-0
        RRCA
        RRCA
        RRCA
        ADD     A,C               ; point BC at digit driver to output
        LD      C,A         
        LD      A,(BC)            ; put driver in A
        OUT     (LDSPLY),A        ; output driver to left display
        LD      C,$0D             ; point BC at driver table again
        LD      A,D               ; A = "counter to display" again
        AND     00001111B         ; A = LSD
        ADD     A,C               ; point BC at digit driver to output
        LD      C,A
        LD      A,(BC)            ; put driver in A
        OUT     (RDSPLY),A        ; output driver to right display
        JP      (IX)              ; jump to continue program

; 
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; 
; "WAIT 1 SEC" DELAY ROUTINE
; 
; ENter with: HL = program "continue address" to jump to

DELAY:
        LD      D,2               ; D=counter1=2
DELAY1:
        LD      A,$FF             ; A=counter2=255
DELAY2:
        LD      B,$FF             ; B=counter3=255
DELAY3:
        DJNZ    DELAY3            ; loop back to DELAY3 255 times
        DEC     A
        JR      NZ,DELAY2         ; loop back to DELAY2 255 times
        DEC     D
        JR      NZ,DELAY1         ; loop back to DELAY1 2 times
        JP      (HL)              ; jump to continue program
; 
; ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; 
; PROGRAM
; 
; SET UP TV DISPLAY
; 
PGM:
        XOR     A                 ; A=0
        OUT     ($08),A           ; set custom chips to run in low-res
        LD      BC,$0818          ; stop any sound
MIN2:   OUT     (C),A
        DJNZ    MIN2              ; (-2)
        LD      A,$CC             ; set vertical blank reg to 204
        OUT     ($0A),A
        LD      A,$14             ; set horiz color bndry to 20, split screen
        OUT     ($09),A
        LD      HL,CTBL           ; set colors
        LD      BC,$080B
        OTIR
        LD      HL,$4000          ; fill top 1/3 of screen with color AA
        LD      BC,$0550
        LD      A,10101010B
MFILL1:
        LD      (HL),A
        CPI
        JP      PE,MFILL1
        LD      BC,$02A8          ; fill next 17 lines with color FF
        LD      A,$FF
MFILL2:
        LD      (HL),A
        CPI
        JP      PE,MFILL2
        LD      A,$55             ; fill top 2 corner bytes with color 55
        LD     ($4000),A
        LD     ($4027),A
; 
; 
; CHECK FOR 2000H SENTINEL
; 
; 
        LD      A,(FIRSTC)        ; jump to 2000H if jmp sentinel C3 present
        CP      $C3
        JP      Z,FIRSTC
; 
; 
; RUN Z80 TO COUNT UP DUAL DISPLAY
; 
; Notes
; Display counts up 00 to FF in hex and repeats nonstop.
; This routine utilizes a nonstandard driver table allowing hex numbers to be
; displayed.
; HEX B is displayed as a lower case b and looks similar to 6.
; Hex D is displayed as a lower case d.
;
        LD      A,01000000B       ; A = dash driver
        OUT     (LDSPLY),A        ; initialize display to - - (dash dash)
        OUT     (RDSPLY),A
        LD      HL,INCD           ; HL = continue address
        JP      DELAY             ; wait 1 sec
INCD:
        LD      E,$00             ; zero the counter
INCD1:
        LD      A,E               ; put counter in A
        LD      IX,INCD2          ; IX = continue address
        JP      DSPLY             ; display the counter digits
INCD2:
        LD      HL,INCD3          ; HL = continue address
        JP      DELAY              ; wait 1 sec
INCD3:
        INC     E                 ; increment counter
        JP      INCD1             ; loop back for next display
; 
; 
; CHECKSUM ADJUSTMENT (to get checksum A4)
; 
        DB      $D5
; 
; 00AB-1FFFH = FF hex
; 
; ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; 
END
 